@@ -87,11 +87,20 @@ else:
         self.isvalue('n',  -1)
 
         self.interp("""
-n=0
+n, i = 0, 0
 while n < 10:
     n += 1
-    if n < 3:
+    if n % 2:
         continue
+    i += 1
+print( 'finish: n, i = ', n, i)
+""")
+        self.isvalue('n',  10)
+        self.isvalue('i',  5)
+
+        self.interp("""
+n=0
+while n < 10:
     n += 1
     print( ' n = ', n)
     if n > 5:
@@ -100,6 +109,30 @@ print( 'finish: n = ', n)
 """)
         self.isvalue('n',  6)
 
+    def test_while_continue(self):
+        self.interp("""
+n, i = 0, 0
+while n < 10:
+    n += 1
+    if n % 2:
+        continue
+    i += 1
+print( 'finish: n, i = ', n, i)
+""")
+        self.isvalue('n',  10)
+        self.isvalue('i',  5)
+
+    def test_while_break(self):
+        self.interp("""
+n = 0
+while n < 10:
+    n += 1
+    if n > 6:
+        break
+print( 'finish: n = ', n)
+""")
+        self.isvalue('n',  7)
+
     def test_assert(self):
         'test assert statements'
         self.interp.error = []
@@ -128,6 +161,7 @@ else:
 ''')
         self.isvalue('n', -1)
 
+    def test_for_break(self):
         self.interp('''
 n=0
 for i in arange(10):
@@ -181,7 +215,8 @@ else:
         self.interp("y = ['a', 'b', 'c']")
         self.interp("rep_x = repr(x['a'])")
         self.interp("rep_y = repr(y)")
-        self.interp("print rep_y , rep_x")
+        self.interp("rep_y , rep_x")
+        self.interp("repr(None)")
 
         self.isvalue("rep_x", "1")
         self.isvalue("rep_y", "['a', 'b', 'c']")
@@ -275,7 +310,7 @@ a = arange(7)''')
             failed, errtype, errmsg = False, None, None
             try:
                 self.interp(expr, show_errors=False)
-            except RuntimeError:
+            except:
                 failed = True
                 errtype, errmsg = self.interp.error[0].get_error()
 
@@ -300,11 +335,10 @@ a = arange(7)''')
     def test_syntaxerrors_2(self):
         '''syntax errors test'''
         for expr in ('x = (1/*)', 'x = 1.A', 'x = A.2'):
-
             failed, errtype, errmsg = False, None, None
             try:
                 self.interp(expr, show_errors=False)
-            except RuntimeError:
+            except: # RuntimeError:
                 failed = True
                 errtype, errmsg = self.interp.error[0].get_error()
             self.assertTrue(failed)
@@ -442,13 +476,18 @@ a = arange(7)''')
         self.interp('x[0:2] = [9,-9]')
         self.isvalue('x', np.array([9,-9,2,3,4,5,6,7,8,9]))
 
-    def test_eservedwords(self):
+    def test_reservedwords(self):
         "test reserved words"
         for w in ('and', 'as', 'while', 'raise', 'else',
                   'class', 'del', 'def', 'import', 'None'):
             self.interp.error= []
-            self.interp("%s= 2" % w)
-            errtype, errmsg = self.interp.error[0].get_error()
+            failed, errtype, errmsg = False, None, None
+            try:
+                self.interp("%s= 2" % w, show_errors=False)
+            except:
+                failed = True
+                errtype, errmsg = self.interp.error[0].get_error()
+
             self.assertTrue(errtype=='SyntaxError')
 
         for w in ('True', 'False'):
@@ -481,7 +520,7 @@ a = arange(7)''')
 x = 5
 try:
     x = x/0
-except ZeroDivsionError:
+except ZeroDivisionError:
     print( 'Error Seen!')
     x = -999
 """)
@@ -491,11 +530,38 @@ except ZeroDivsionError:
 x = -1
 try:
     x = x/0
-except ZeroDivsionError:
+except ZeroDivisionError:
     pass
 """)
         self.isvalue('x', -1)
 
+
+    def test_tryelsefinally(self):
+
+        self.interp("""def dotry(x, y):
+    out, ok, clean = 0, False, False
+    try:
+        out = x/y
+    except ZeroDivisionError:
+        out = -1
+    else:
+        ok = True
+    finally:
+        clean = True
+    return out, ok, clean
+""")
+        self.interp("val, ok, clean = dotry(1, 2.0)")
+        self.interp("print(ok, clean)")
+        self.isnear("val", 0.5)
+        self.isvalue("ok", True)
+        self.isvalue("clean", True)
+
+        self.interp("val, ok, clean = dotry(1, 0.0)")
+        self.isvalue("val", -1)
+        self.isvalue("ok",  False)
+        self.isvalue("clean", True)
+
+
     def test_function1(self):
         "test function definition and running"
         self.interp("""
